@@ -1,22 +1,15 @@
 use core::{marker::PhantomData, pin::Pin};
 
-use std::{collections::HashMap, sync::{Arc, atomic::{AtomicU64, Ordering}}};
+use std::{collections::HashSet, sync::{Arc, atomic::{AtomicU64, Ordering}}};
 
 use futures::{Future, FutureExt};
-use smallvec::SmallVec;
 use tokio::sync::Mutex;
 
-use crate::{AsyncBatchHandler, AsyncBatchSynchronizedHandler, AsyncHandler, AsyncSynchronizedHandler, BatchHandler, BatchSynchronizedHandler, Bus, BusInner, Handler, Message, Relay, SharedMessage, SynchronizedHandler, Untyped, envelop::{IntoSharedMessage, TypeTag}, error::{Error, StdSyncSendError}, receiver::{
+use crate::{AsyncBatchHandler, AsyncBatchSynchronizedHandler, AsyncHandler, AsyncSynchronizedHandler, BatchHandler, BatchSynchronizedHandler, Bus, BusInner, Handler, Message, Relay, SynchronizedHandler, Untyped, error::StdSyncSendError, receiver::{
         BusPollerCallback, Receiver, ReciveTypedReceiver, SendTypedReceiver, SendUntypedReceiver,
         UntypedPollerCallback,
     }, receivers};
 
-type MessageDeserializerCallback = Box<
-    dyn Fn(&mut dyn erased_serde::Deserializer<'_>) -> Result<Box<dyn SharedMessage>, Error>
-        + Send
-        + Sync,
->;
-
 static RECEVIER_ID_SEQ: AtomicU64 = AtomicU64::new(1);
 
 pub trait ReceiverSubscriberBuilder<T, M, R, E>:
@@ -43,19 +36,19 @@ pub struct RegisterEntry<K, T, F, P, B> {
     payload: B,
     builder: F,
     poller: P,
-    receivers: HashMap<TypeTag, Receiver>,
+    receivers: HashSet<Receiver>,
     pollers: Vec<BusPollerCallback>,
     _m: PhantomData<(K, T)>,
 }
 
 impl<K, T: 'static, F, P, B> RegisterEntry<K, T, F, P, B>
 where
-    F: FnMut(&mut B, TypeTag, Receiver),
+    F: FnMut(&mut B, Receiver),
     P: FnMut(&mut B, BusPollerCallback),
 {
     pub fn done(mut self) -> B {
-        for (tid, v) in self.receivers {
-            (self.builder)(&mut self.payload, tid, v);
+        for r in self.receivers {
+            (self.builder)(&mut self.payload, r);
         }
 
         for p in self.pollers {
@@ -77,9 +70,9 @@ impl<T, F, P, B> RegisterEntry<UnsyncEntry, T, F, P, B> {
     {
         let (inner, poller) = S::build(cfg);
 
-        let receiver = Receiver::new::<M, R, E, S>(RECEVIER_ID_SEQ.fetch_add(1, Ordering::Relaxed), queue, inner);
+        let receiver = Receiver::new::<M, R, E, S>(RECEVIER_ID_SEQ.fetch_add(1, Ordering::Relaxed), queue, true, inner);
         let poller2 = receiver.start_polling();
-        self.receivers.insert(M::type_tag_(), receiver);
+        self.receivers.insert(receiver);
         self.pollers.push(poller(self.item.clone()));
         self.pollers.push(poller2);
 
@@ -146,9 +139,9 @@ impl<T, F, P, B> RegisterEntry<SyncEntry, T, F, P, B> {
     {
         let (inner, poller) = S::build(cfg);
 
-        let receiver = Receiver::new::<M, R, E, S>(RECEVIER_ID_SEQ.fetch_add(1, Ordering::Relaxed), queue, inner);
+        let receiver = Receiver::new::<M, R, E, S>(RECEVIER_ID_SEQ.fetch_add(1, Ordering::Relaxed), queue, true, inner);
         let poller2 = receiver.start_polling();
-        self.receivers.insert(M::type_tag_(), receiver);
+        self.receivers.insert(receiver);
         self.pollers.push(poller(self.item.clone()));
         self.pollers.push(poller2);
 
@@ -205,77 +198,24 @@ impl<T, F, P, B> RegisterEntry<SyncEntry, T, F, P, B> {
     }
 }
 
-pub struct MessageTypeDescriptor {
-    de: MessageDeserializerCallback,
-}
-
-impl MessageTypeDescriptor {
-    #[inline]
-    pub fn deserialize_boxed(
-        &self,
-        de: &mut dyn erased_serde::Deserializer<'_>,
-    ) -> Result<Box<dyn SharedMessage>, Error> {
-        (self.de)(de)
-    }
-}
-
 #[derive(Default)]
 pub struct Module {
-    message_types: HashMap<TypeTag, MessageTypeDescriptor>,
-    receivers: HashMap<TypeTag, SmallVec<[Receiver; 4]>>,
+    receivers: HashSet<Receiver>,
     pollings: Vec<BusPollerCallback>,
 }
 
 impl Module {
     pub fn new() -> Self {
         Self {
-            message_types: HashMap::new(),
-            receivers: HashMap::new(),
+            receivers: HashSet::new(),
             pollings: Vec::new(),
         }
     }
 
-    pub fn register_shared_message<
-        M: Message + Clone + serde::Serialize + serde::de::DeserializeOwned,
-    >(
-        mut self,
-    ) -> Self {
-        println!("insert {}", M::type_tag_());
-        self.message_types.insert(
-            M::type_tag_(),
-            MessageTypeDescriptor {
-                de: Box::new(move |de| Ok(M::deserialize(de)?.into_shared())),
-            },
-        );
-
-        self
-    }
-
     pub fn register_relay<S: Relay + Send + Sync + 'static>(mut self, inner: S) -> Self {
         let receiver = Receiver::new_relay::<S>(RECEVIER_ID_SEQ.fetch_add(1, Ordering::Relaxed), inner);
         self.pollings.push(receiver.start_polling());
-
-        let mut receiver_added = false;
-        receiver.iter_types(&mut |msg, _, _| {
-            self.receivers
-                .entry(msg.clone())
-                .or_insert_with(SmallVec::new)
-                .push(receiver.clone());
-
-            if !receiver_added {
-                receiver_added = true;
-                false
-            } else {
-                true
-            }
-        });
-
-        if !receiver_added {
-            self.receivers
-                .entry("".into())
-                .or_insert_with(SmallVec::new)
-                .push(receiver);
-        }
+        self.receivers.insert(receiver);
 
         self
     }
@@ -286,18 +226,16 @@ impl Module {
     ) -> RegisterEntry<
         SyncEntry,
         T,
-        impl FnMut(&mut Self, TypeTag, Receiver),
+        impl FnMut(&mut Self, Receiver),
         impl FnMut(&mut Self, Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>),
         Self,
     > {
         RegisterEntry {
             item: Arc::new(item) as Untyped,
             payload: self,
-            builder: |p: &mut Self, tt, r| {
-                p.receivers.entry(tt).or_insert_with(SmallVec::new).push(r);
-            },
+            builder: |p: &mut Self, r| { p.receivers.insert(r); },
             poller: |p: &mut Self, poller| p.pollings.push(poller),
-            receivers: HashMap::new(),
+            receivers: HashSet::new(),
             pollers: Vec::new(),
             _m: Default::default(),
         }
@@ -309,7 +247,7 @@ impl Module {
     ) -> RegisterEntry<
         UnsyncEntry,
         T,
-        impl FnMut(&mut Self, TypeTag, Receiver),
+        impl FnMut(&mut Self, Receiver),
         impl FnMut(&mut Self, Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>),
         Self,
     > {
@@ -318,26 +256,17 @@ impl Module {
         RegisterEntry {
             item,
             payload: self,
-            builder: |p: &mut Self, tt, r| {
-                p.receivers.entry(tt).or_insert_with(SmallVec::new).push(r);
-            },
+            builder: |p: &mut Self, r| { p.receivers.insert(r); },
             poller: |p: &mut Self, poller| p.pollings.push(poller),
-            receivers: HashMap::new(),
+            receivers: HashSet::new(),
             pollers: Vec::new(),
             _m: Default::default(),
         }
     }
 
     pub fn add_module(mut self, module: Module) -> Self {
-        self.message_types.extend(module.message_types);
         self.pollings.extend(module.pollings);
-
-        for (k, v) in module.receivers {
-            self.receivers
-                .entry(k)
-                .or_insert_with(Default::default)
-                .extend(v);
-        }
+        self.receivers.extend(module.receivers);
 
         self
     }
@@ -354,16 +283,6 @@ impl BusBuilder {
         }
     }
 
-    pub fn register_shared_message<
-        M: Message + Clone + serde::Serialize + serde::de::DeserializeOwned,
-    >(
-        self,
-    ) -> Self {
-        let inner = self.inner.register_shared_message::<M>();
-
-        BusBuilder { inner }
-    }
-
     pub fn register_relay<S: Relay + Send + Sync + 'static>(self, inner: S) -> Self {
         let inner = self.inner.register_relay(inner);
 
@@ -376,22 +295,16 @@ impl BusBuilder {
     ) -> RegisterEntry<
         SyncEntry,
         T,
-        impl FnMut(&mut Self, TypeTag, Receiver),
+        impl FnMut(&mut Self, Receiver),
         impl FnMut(&mut Self, Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>),
         Self,
     > {
         RegisterEntry {
             item: Arc::new(item) as Untyped,
             payload: self,
-            builder: |p: &mut Self, tt, r| {
-                p.inner
-                    .receivers
-                    .entry(tt)
-                    .or_insert_with(SmallVec::new)
-                    .push(r);
-            },
+            builder: |p: &mut Self, r| {  p.inner.receivers.insert(r); },
             poller: |p: &mut Self, poller| p.inner.pollings.push(poller),
-            receivers: HashMap::new(),
+            receivers: HashSet::new(),
             pollers: Vec::new(),
             _m: Default::default(),
         }
@@ -403,22 +316,16 @@ impl BusBuilder {
     ) -> RegisterEntry<
         UnsyncEntry,
         T,
-        impl FnMut(&mut Self, TypeTag, Receiver),
+        impl FnMut(&mut Self, Receiver),
         impl FnMut(&mut Self, Box<dyn FnOnce(Bus) -> Pin<Box<dyn Future<Output = ()> + Send>>>),
         Self,
     > {
         RegisterEntry {
             item: Arc::new(Mutex::new(item)) as Untyped,
             payload: self,
-            builder: |p: &mut Self, tt, r| {
-                p.inner
-                    .receivers
-                    .entry(tt)
-                    .or_insert_with(SmallVec::new)
-                    .push(r);
-            },
+            builder: |p: &mut Self, r| { p.inner.receivers.insert(r); },
             poller: |p: &mut Self, poller| p.inner.pollings.push(poller),
-            receivers: HashMap::new(),
+            receivers: HashSet::new(),
             pollers: Vec::new(),
             _m: Default::default(),
         }
@@ -431,19 +338,8 @@ impl BusBuilder {
     }
 
     pub fn build(self) -> (Bus, impl Future<Output = ()>) {
-        let mut receivers = HashMap::new();
-
-        for (key, values) in self.inner.receivers {
-            for v in values {
-                receivers
-                    .entry(key.clone())
-                    .or_insert_with(SmallVec::new)
-                    .push(v);
-            }
-        }
-
         let bus = Bus {
-            inner: Arc::new(BusInner::new(receivers, self.inner.message_types)),
+            inner: Arc::new(BusInner::new(self.inner.receivers)),
         };
 
         let mut futs = Vec::with_capacity(self.inner.pollings.len() * 2);
