@@ -7,6 +7,13 @@ pub mod receivers;
 mod relay;
 mod stats;
 mod trait_object;
+pub mod type_tag;
+
+
+pub mod __reexport {
+    pub use serde;
+    pub use ctor;
+}
 
 #[macro_use]
 extern crate log;
@@ -22,10 +29,10 @@ use core::{
     time::Duration,
 };
 use smallvec::SmallVec;
-use std::{collections::HashMap, sync::Arc};
+use std::{collections::{HashMap, HashSet}, sync::Arc};
 use tokio::sync::Mutex;
 
-use builder::{BusBuilder, MessageTypeDescriptor};
+use builder::BusBuilder;
 use error::{Error, SendError, StdSyncSendError};
 use receiver::{Permit, Receiver};
 use stats::Stats;
@@ -35,13 +42,16 @@ pub use builder::Module;
 pub use envelop::{IntoBoxedMessage, Message, MessageBounds, SharedMessage, TypeTag, TypeTagged};
 pub use handler::*;
 pub use receiver::{
-    Action, Event, ReciveTypedReceiver, ReciveUntypedReceiver, SendTypedReceiver,
+    Action, Event, EventBoxed, ReciveTypedReceiver, ReciveUntypedReceiver, SendTypedReceiver,
     SendUntypedReceiver, TypeTagAccept,
 };
 pub use relay::Relay;
-
+pub use ctor;
+pub use type_tag::{register_shared_message, deserialize_shared_message};
 pub type Untyped = Arc<dyn Any + Send + Sync>;
 
+type LookupQuery = (TypeTag, Option<TypeTag>, Option<TypeTag>);
+
 static ID_COUNTER: AtomicU64 = AtomicU64::new(1);
 
 #[derive(Debug, Clone, Copy, PartialEq)]
@@ -60,20 +70,47 @@ impl Default for SendOptions {
 }
 
 pub struct BusInner {
-    message_types: HashMap<TypeTag, MessageTypeDescriptor>,
-    receivers: HashMap<TypeTag, SmallVec<[Receiver; 4]>>,
+    receivers: HashSet<Receiver>,
+    lookup: HashMap<LookupQuery, SmallVec<[Receiver; 4]>>,
     closed: AtomicBool,
     maintain: Mutex<()>,
 }
 
 impl BusInner {
     pub(crate) fn new(
-        receivers: HashMap<TypeTag, SmallVec<[Receiver; 4]>>,
-        message_types: HashMap<TypeTag, MessageTypeDescriptor>,
+        receivers: HashSet<Receiver>,
     ) -> Self {
+        let mut lookup = HashMap::new();
+        for recv in receivers.iter() {
+            for (msg, resp) in recv.iter_types() {
+                lookup.entry((msg.clone(), None, None))
+                    .or_insert_with(HashSet::new)
+                    .insert(recv.clone());
+
+                if let Some((resp, err)) = resp {
+                    lookup.entry((msg.clone(), Some(resp.clone()), None))
+                        .or_insert_with(HashSet::new)
+                        .insert(recv.clone());
+
+                    lookup.entry((msg.clone(), None, Some(err.clone())))
+                        .or_insert_with(HashSet::new)
+                        .insert(recv.clone());
+
+                    lookup.entry((msg, Some(resp), Some(err)))
+                        .or_insert_with(HashSet::new)
+                        .insert(recv.clone());
+                }
+            }
+        }
+
+        let lookup = lookup
+            .into_iter()
+            .map(|(k, v)| (k, v.into_iter().collect()))
+            .collect();
+
         Self {
-            message_types,
             receivers,
+            lookup,
             closed: AtomicBool::new(false),
             maintain: Mutex::new(()),
         }
@@ -96,18 +133,14 @@ impl Bus {
     }
 
     pub(crate) fn init(&self) {
-        for rs in self.inner.receivers.values() {
-            for r in rs {
-                r.init(self).unwrap();
-            }
+        for r in self.inner.receivers.iter() {
+            r.init(self).unwrap();
         }
     }
 
     pub async fn ready(&self) {
-        for rs in self.inner.receivers.values() {
-            for r in rs {
-                r.ready().await;
-            }
+        for r in self.inner.receivers.iter() {
+            r.ready().await;
         }
     }
 
@@ -115,13 +148,11 @@ impl Bus {
         let _handle = self.inner.maintain.lock().await;
         self.inner.closed.store(true, Ordering::SeqCst);
 
-        for rs in self.inner.receivers.values() {
-            for r in rs {
-                let err = tokio::time::timeout(Duration::from_secs(20), r.close(self)).await;
+        for r in self.inner.receivers.iter() {
+            let err = tokio::time::timeout(Duration::from_secs(20), r.close(self)).await;
 
-                if let Err(err) = err {
-                    error!("Close timeout on {}: {}", r.name(), err);
-                }
+            if let Err(err) = err {
+                error!("Close timeout on {}: {}", r.name(), err);
             }
         }
     }
@@ -134,13 +165,11 @@ impl Bus {
         for _ in 0..fuse_count {
             iters += 1;
             let mut flushed = false;
-            for rs in self.inner.receivers.values() {
-                for r in rs {
-                    if r.need_flush() {
-                        flushed = true;
+            for r in self.inner.receivers.iter() {
+                if r.need_flush() {
+                    flushed = true;
 
-                        r.flush(self).await;
-                    }
+                    r.flush(self).await;
                 }
             }
 
@@ -163,10 +192,8 @@ impl Bus {
     pub async fn sync(&self) {
         let _handle = self.inner.maintain.lock().await;
 
-        for rs in self.inner.receivers.values() {
-            for r in rs {
-                r.sync(self).await;
-            }
+        for r in self.inner.receivers.iter() {
+            r.sync(self).await;
         }
     }
 
@@ -207,7 +234,7 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt) {
+        if let Some(rs) = self.inner.lookup.get(&(msg.type_tag(), None, None)) {
             let permits = if let Some(x) = self.try_reserve(&tt, rs) {
                 x
             } else {
@@ -270,7 +297,7 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt) {
+        if let Some(rs) = self.inner.lookup.get(&(msg.type_tag(), None, None)) {
             if let Some((last, head)) = rs.split_last() {
                 for r in head {
                     let _ = r.send(self, mid, msg.clone(), false, r.reserve(&tt).await);
@@ -304,10 +331,9 @@ impl Bus {
             return Err(SendError::Closed(msg).into());
         }
 
-        let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt) {
+        if let Some(rs) = self.inner.lookup.get(&(msg.type_tag(), None, None)) {
             if let Some((last, head)) = rs.split_last() {
                 for r in head {
                     let _ = r.force_send(self, mid, msg.clone(), false);
@@ -336,7 +362,9 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt).and_then(|rs| rs.first()) {
+        if let Some(rs) = self.inner.lookup.get(&(msg.type_tag(), None, None))
+            .and_then(|rs| rs.first()) 
+        {
             let permits = if let Some(x) = rs.try_reserve(&tt) {
                 x
             } else {
@@ -357,7 +385,9 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt).and_then(|rs| rs.first()) {
+        if let Some(rs) = self.inner.lookup.get(&(msg.type_tag(), None, None))
+            .and_then(|rs| rs.first()) 
+        {
             Ok(rs.send(self, mid, msg, false, rs.reserve(&tt).await)?)
         } else {
             Err(Error::NoReceivers)
@@ -374,10 +404,10 @@ impl Bus {
         req: M,
         options: SendOptions,
     ) -> Result<R, Error<M>> {
-        let tid = req.type_tag();
+        let tid = M::type_tag_();
         let rid = R::type_tag_();
 
-        let mut iter = self.select_receivers(&tid, options, Some(&rid), None);
+        let mut iter = self.select_receivers(tid.clone(), options, Some(rid), None, true);
         if let Some(rc) = iter.next() {
             let (mid, rx) = rc
                 .add_response_waiter::<R>()
@@ -402,7 +432,7 @@ impl Bus {
         let rid = R::type_tag_();
         let eid = E::type_tag_();
 
-        let mut iter = self.select_receivers(&tid, options, Some(&rid), Some(&eid));
+        let mut iter = self.select_receivers(tid.clone(), options, Some(rid), Some(eid), true);
         if let Some(rc) = iter.next() {
             let (mid, rx) = rc.add_response_waiter_we::<R, E>().map_err(|x| {
                 x.map_err(|_| unimplemented!())
@@ -436,7 +466,7 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        let mut iter = self.select_receivers(&tt, options, None, None);
+        let mut iter = self.select_receivers(tt.clone(), options, None, None, false);
         let first = iter.next();
 
         for r in iter {
@@ -476,7 +506,7 @@ impl Bus {
         let tt = msg.type_tag();
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        let mut iter = self.select_receivers(&tt, options, None, None);
+        let mut iter = self.select_receivers(tt.clone(), options, None, None, false);
         if let Some(rs) = iter.next() {
             Ok(rs.send_boxed(self, mid, msg, false, rs.reserve(&tt).await)?)
         } else {
@@ -495,7 +525,7 @@ impl Bus {
 
         let tt = req.type_tag();
 
-        let mut iter = self.select_receivers(&tt, options, None, None);
+        let mut iter = self.select_receivers(tt.clone(), options, None, None, true);
         if let Some(rc) = iter.next() {
             let (mid, rx) = rc.add_response_waiter_boxed().map_err(|x| {
                 x.map_err(|_| unimplemented!())
@@ -528,7 +558,7 @@ impl Bus {
         let tt = req.type_tag();
         let eid = E::type_tag_();
 
-        let mut iter = self.select_receivers(&tt, options, None, Some(&eid));
+        let mut iter = self.select_receivers(tt.clone(), options, None, Some(eid), true);
         if let Some(rc) = iter.next() {
             let (mid, rx) = rc.add_response_waiter_boxed_we().map_err(|x| {
                 x.map_err(|_| unimplemented!())
@@ -563,8 +593,11 @@ impl Bus {
 
         let mid = ID_COUNTER.fetch_add(1, Ordering::Relaxed);
 
-        if let Some(rs) = self.inner.receivers.get(&tt).and_then(|rs| rs.first()) {
-            let msg = self.deserialize_message(tt.clone(), de)?;
+        if let Some(rs) = self.inner.lookup.get(&(tt.clone(), None, None))
+        .and_then(|rs| rs.first()) {
+            
+            let msg = deserialize_shared_message(tt.clone(), de)?;
+            
             Ok(rs.send_boxed(self, mid, msg.upcast_box(), false, rs.reserve(&tt).await)?)
         } else {
             Err(Error::NoReceivers)
@@ -582,10 +615,10 @@ impl Bus {
             return Err(Error::NoResponse);
         }
 
-        let mut iter = self.select_receivers(&tt, options, None, None);
+        let mut iter = self.select_receivers(tt.clone(), options, None, None, true);
         if let Some(rc) = iter.next() {
             let (mid, rx) = rc.add_response_waiter_boxed().unwrap();
-            let msg = self.deserialize_message(tt.clone(), de)?;
+            let msg = deserialize_shared_message(tt.clone(), de)?;
 
             rc.send_boxed(
                 self,
@@ -601,44 +634,26 @@ impl Bus {
         }
     }
 
-    pub fn deserialize_message(
-        &self,
-        tt: TypeTag,
-        de: &mut dyn erased_serde::Deserializer<'_>,
-    ) -> Result<Box<dyn SharedMessage>, Error<Box<dyn Message>>> {
-        let md = self
-            .inner
-            .message_types
-            .get(&tt)
-            .ok_or(Error::TypeTagNotRegistered(tt))?;
-
-        md.deserialize_boxed(de)
-            .map_err(|err| err.specify::<Box<dyn Message>>())
-    }
-
     pub fn stats(&self) -> impl Iterator<Item = Stats> + '_ {
         self.inner
             .receivers
             .iter()
-            .map(|(_, r)| r.into_iter().map(|x| x.stats()))
-            .flatten()
+            .map(|x| x.stats())
     }
 
     #[inline]
-    fn select_receivers<'a, 'b: 'a, 'c: 'a, 'd: 'a>(
-        &'a self,
-        tid: &'b TypeTag,
+    fn select_receivers(
+        &self,
+        tid: TypeTag,
         options: SendOptions,
-        rid: Option<&'c TypeTag>,
-        eid: Option<&'d TypeTag>,
-    ) -> impl Iterator<Item = &Receiver> + 'a {
-        self.inner
-            .receivers
-            .get(tid)
+        rid: Option<TypeTag>,
+        eid: Option<TypeTag>,
+        is_req: bool,
+    ) -> impl Iterator<Item = &Receiver> + '_ {
+       self.inner.lookup.get(&(tid.clone(), rid.clone(), eid.clone()))
             .into_iter()
-            .map(|item| item.iter())
             .flatten()
-            .filter(move |r| r.accept(tid, rid, eid))
+            .filter(move |r| r.accept(is_req, &tid, rid.as_ref(), eid.as_ref()))
             .filter(move |r| match options {
                 SendOptions::Except(id) => id != r.id(),
                 SendOptions::Direct(id) => id == r.id(),
