@@ -6,7 +6,7 @@ use std::sync::{
 use async_trait::async_trait;
 use messagebus::{
     derive::{Error as MbError, Message},
-    error, AsyncHandler, Bus, Message, MessageBounds,
+    error, AsyncHandler, Bus, Message,
 };
 use thiserror::Error;
 
@@ -22,14 +22,13 @@ impl<M: Message> From<error::Error<M>> for Error {
     }
 }
 
-#[derive(Debug, Clone, Message)]
+#[derive(Debug, Clone, serde_derive::Serialize, serde_derive::Deserialize, Message)]
+#[message(clone, shared)]
 struct Msg;
 
 #[derive(Debug, Clone, serde_derive::Serialize, serde_derive::Deserialize, Message)]
 #[message(clone, shared)]
-struct SharedMsg<T: serde::Serialize + MessageBounds + Clone>(
-    #[serde(bound(deserialize = "T: serde::Deserialize<'de>"))] T,
-);
+struct SharedMsg<T>(T);
 
 struct TmpReceiverContext {
     sync1: AtomicBool,
